using System;
using System.Data;
using System.Transactions;
using System.Data.SqlClient;
using System.Configuration;
using System.Configuration.Provider;
using System.Web;
using System.Web.Configuration;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

public class ExtendedMembershipDataProvider : SqlMembershipProvider
{
    private string connectionString;

	public ExtendedMembershipDataProvider() {}

    public override void Initialize(string name, System.Collections.Specialized.NameValueCollection config)
    {
        //Get the connection string from configuration
        string connectionStringName = config["connectionStringName"];
        string tempConn;
        if (String.IsNullOrEmpty(connectionStringName))
            throw new ProviderException("You must specify connectionStringName in the provider configuration");

        ConnectionStringsSection cs =
            (ConnectionStringsSection)WebConfigurationManager.GetSection("connectionStrings");
        if (cs == null)
            throw new ProviderException("An error occurred retrieving the connection strings section.");

        if (cs.ConnectionStrings[connectionStringName] == null)
            throw new ProviderException("The connection string could not be found in the connection strings section.");
        else
            tempConn = cs.ConnectionStrings[connectionStringName].ConnectionString;

        if (String.IsNullOrEmpty(tempConn))
            throw new ProviderException("The connection string is invalid.");

        //Don't remove the connection string at this point because the base provider needs it too
        base.Initialize(name, config);

        //Now it is safe to store the connection string
        connectionString = tempConn;
    }

    public override MembershipUser CreateUser(string username, string password, string email, string passwordQuestion, string passwordAnswer, bool isApproved, object providerUserKey, out MembershipCreateStatus status)
    {
        TransactionOptions to = new TransactionOptions();
        to.Timeout = new TimeSpan(0,0,30);
        to.IsolationLevel = System.Transactions.IsolationLevel.ReadCommitted;

        using (TransactionScope scope =
                    new TransactionScope(TransactionScopeOption.Required, to))
        {
            MembershipUser user =
                base.CreateUser(username, password, email, passwordQuestion, passwordAnswer, isApproved, providerUserKey, out status);

            if (status == MembershipCreateStatus.Success)
            {
                using (SqlConnection conn = new SqlConnection(connectionString))
                {
                    HttpContext c = HttpContext.Current;
                    string firstName = (string)c.Items["First Name"];
                    string lastName = (string)c.Items["Last Name"];

                    SqlCommand cmd = new SqlCommand("CreateExtendedData", conn);
                    cmd.CommandType = CommandType.StoredProcedure;
                    SqlParameter p1 = new SqlParameter("@pApplicationName", this.ApplicationName);
                    SqlParameter p2 = new SqlParameter("@pUserName", username);
                    SqlParameter p3 = new SqlParameter("@pFirstName", firstName);
                    SqlParameter p4 = new SqlParameter("@pLastName", lastName);
                    SqlParameter p5 = new SqlParameter("@pReturnValue", SqlDbType.Int);
                    p5.Direction = ParameterDirection.ReturnValue;
                    cmd.Parameters.AddRange(new SqlParameter[] { p1, p2, p3, p4, p5 });

                    conn.Open();
                    cmd.ExecuteNonQuery();
                    int retVal = (int)p5.Value;
                    conn.Close();

                    if (retVal != 0)
                    {
                        status = MembershipCreateStatus.ProviderError;
                        return null;
                    }
                    else
                    {
                        //Commit both pieces of work
                        scope.Complete();

                        //Return an extended membership user
                        ExtendedMembershipUser emu = new ExtendedMembershipUser(user);
                        emu.FirstName = firstName;
                        emu.LastName = lastName;

                        return emu;
                    }
                }
            }
            else
            {
                return null;
            }
        }
    }

    public override MembershipUser GetUser(string username, bool userIsOnline)
    {
        MembershipUser mu = base.GetUser(username, userIsOnline);
        if (mu != null)
        {
            using (SqlConnection conn = new SqlConnection(connectionString))
            {
                string firstName = null;
                string lastName = null;

                SqlCommand cmd = new SqlCommand("GetExtendedData", conn);
                cmd.CommandType = CommandType.StoredProcedure;
                SqlParameter p1 = new SqlParameter("@pApplicationName", this.ApplicationName);
                SqlParameter p2 = new SqlParameter("@pUserName", username);
                cmd.Parameters.AddRange(new SqlParameter[] { p1, p2});

                conn.Open();
                SqlDataReader dr = cmd.ExecuteReader();
                if (dr.Read())
                {
                    //First Name
                    if (dr.IsDBNull(0))
                        firstName = null;
                    else
                        firstName = dr.GetString(0);

                    //Last Name
                    if (dr.IsDBNull(1))
                        lastName = null;
                    else
                        lastName = dr.GetString(1);
                }

                    conn.Close();

                    //Return an extended membership user
                    ExtendedMembershipUser emu = new ExtendedMembershipUser(mu);
                    emu.FirstName = firstName;
                    emu.LastName = lastName;

                    return emu;
            }
        }
        else
            return mu;
    }

    public override bool DeleteUser(string username, bool deleteAllRelatedData)
    {
        TransactionOptions to = new TransactionOptions();
        to.Timeout = new TimeSpan(0,0,30);
        to.IsolationLevel = System.Transactions.IsolationLevel.ReadCommitted;

        using (TransactionScope scope =
                    new TransactionScope(TransactionScopeOption.Required, to))
        {
            //first delete the extended data
            using (SqlConnection conn = new SqlConnection(connectionString))
            {
                try
                {
                    SqlCommand cmd = new SqlCommand("DeleteExtendedData", conn);
                    cmd.CommandType = CommandType.StoredProcedure;
                    SqlParameter p1 = new SqlParameter("@pApplicationName", this.ApplicationName);
                    SqlParameter p2 = new SqlParameter("@pUserName", username);
                    cmd.Parameters.AddRange(new SqlParameter[] { p1, p2 });

                    conn.Open();
                    cmd.ExecuteNonQuery();
                    conn.Close();

                }
                catch
                {
                    return false;
                }
            }

            //If the base class call succeeds as well, then commit all the work
            bool  retval = base.DeleteUser(username, deleteAllRelatedData);
            if (retval)
                scope.Complete();

            return retval;
        }//end global tran
        
    }

    public override void UpdateUser(MembershipUser user)
    {
        ExtendedMembershipUser emu;

        if (!(user is ExtendedMembershipUser))
            throw new ArgumentException("You must supply an object of type ExtendedMembershipUser.");

        emu = (ExtendedMembershipUser)user;

        TransactionOptions to = new TransactionOptions();
        to.Timeout = new TimeSpan(0,0,30);
        to.IsolationLevel = System.Transactions.IsolationLevel.ReadCommitted;

        using (TransactionScope scope =
                    new TransactionScope(TransactionScopeOption.Required, to))
        {
            //First update the base user record
            base.UpdateUser(user);

            //Then update the extended data
            using (SqlConnection conn = new SqlConnection(connectionString))
            {
                SqlCommand cmd = new SqlCommand("UpdateExtendedData", conn);
                cmd.CommandType = CommandType.StoredProcedure;
                SqlParameter p1 = new SqlParameter("@pApplicationName", this.ApplicationName);
                SqlParameter p2 = new SqlParameter("@pUserName", emu.UserName);
                SqlParameter p3 = new SqlParameter("@pFirstName", emu.FirstName);
                SqlParameter p4 = new SqlParameter("@pLastName", emu.LastName);
                SqlParameter p5 = new SqlParameter("@pReturnValue", SqlDbType.Int);
                p5.Direction = ParameterDirection.ReturnValue;
                cmd.Parameters.AddRange(new SqlParameter[] { p1, p2, p3, p4, p5 });

                conn.Open();
                cmd.ExecuteNonQuery();
                int retVal = (int)p5.Value;
                conn.Close();

                if (retVal != 0)
                {
                    //Can't really undo anything at this point
                    //Just throw error
                    throw new ProviderException("An error occurred while attempting to update the extended data.");
                }
            }

            //Made it this far - so commit the work
            scope.Complete();

        }//end global tran
    }

}
