drop table StagingConfigurations
drop table MachineApplicationToEnvironment
drop table Features
drop table StagingEnvironments
drop table Machines
drop table Applications
go

--
--Defines available staging environments
--
create table StagingEnvironments(
StagingEnvironmentID int not null IDENTITY PRIMARY KEY,
StagingEnvironmentName nvarchar(256) not null
)
go
alter table StagingEnvironments add constraint
 UQ_StagingEnvironmentName UNIQUE (StagingEnvironmentName)
go

--
--Define available features that can be configured
--
create table Features (
FeatureID int not null IDENTITY PRIMARY KEY,
FeatureName nvarchar(256) not null)
go
alter table Features add constraint
 UQ_FeatureName UNIQUE (FeatureName)
go

--
--Machines that can be registered 
--
create table Machines (
MachineID int not null IDENTITY PRIMARY KEY,
MachineName nvarchar(256) not null
)
go
alter table Machines add constraint
 UQ_MachineName UNIQUE (MachineName)
go

--
--Applications that can be registered
--
create table Applications (
ApplicationID int not null IDENTITY PRIMARY KEY,
ApplicationName nvarchar(256) not null
)
go
alter table Applications add constraint
 UQ_ApplicationName UNIQUE (ApplicationName)
go

--
--Maps a an application running on a machine
--to a specific deployment environment
--
create table MachineApplicationToEnvironment (
MachineID int not null,
ApplicationID int not null,
StagingEnvironmentID int not null)
go
alter table MachineApplicationToEnvironment add constraint
  PK_MachineApplicationToEnvironment PRIMARY KEY (MachineID, ApplicationID)
go
alter table MachineApplicationToEnvironment add constraint
  FK_MAE_to_Machines FOREIGN KEY (MachineID)
  REFERENCES Machines(MachineID)
go
alter table MachineApplicationToEnvironment add constraint
  FK_MAE_to_Applications FOREIGN KEY (ApplicationID)
  REFERENCES Applications(ApplicationID)
go

--
--Stores the configuration data for a single configuration section
--for a specific staging environment
--
create table StagingConfigurations (
ConfigurationID	int not null IDENTITY PRIMARY KEY,
ApplicationID int not null,
StagingEnvironmentID int not null,
FeatureID int not null,
FeatureConfiguration ntext)
go
alter table StagingConfigurations add constraint
  UQ_MachineApplicationToEnvironment UNIQUE (ApplicationID, FeatureID, StagingEnvironmentID)
go

alter table StagingConfigurations add constraint
  FK_to_StagingEnvironments FOREIGN KEY (StagingEnvironmentID)
  REFERENCES StagingEnvironments(StagingEnvironmentID)
go

alter table StagingConfigurations add constraint
  FK_to_Features FOREIGN KEY (FeatureID)
  REFERENCES Features(FeatureID)
go

alter table StagingConfigurations add constraint
  FK_to_Applications FOREIGN KEY (ApplicationID)
  REFERENCES Applications(ApplicationID)
go

drop procedure GetConfiguration
go
create procedure GetConfiguration
@pMachineName 		nvarchar(256),
@pApplicationName	nvarchar(256),
@pFeatureName       nvarchar(256)
as
begin
    declare @machineID        int
    declare @featureID        int
    declare @applicationID    int

    select @machineID = MachineID
    from    Machines
    where   MachineName = @pMachineName

    select @featureID = FeatureID
    from    Features
    where   FeatureName = @pFeatureName

    select @applicationID = ApplicationID
    from    Applications
    where   ApplicationName = @pApplicationName

    select FeatureConfiguration
    from   MachineApplicationToEnvironment mae,
           StagingConfigurations sc
    where  mae.MachineID            = @machineID
    and    mae.ApplicationID        = @applicationID
    and    mae.StagingEnvironmentID = sc.StagingEnvironmentID
    and    sc.ApplicationID         = @applicationID
    and    sc.FeatureID             = @featureID
end
go

--grant rights on the sproc to the appropriate account(s)
grant execute on GetConfiguration to [NT AUTHORITY\NETWORK SERVICE]
go
